import 'package:flutter/material.dart';
import 'dart:ui';

enum Shape {
  SQUARE,
  RECTANGLE,
  CIRCLE,
}

enum Fill {
  IN,
  OUT,
}

abstract class FocusShape {
  final Shape shape;
  final Fill fill;
  FocusShape(this.shape, Fill fill) :
    this.fill = fill ?? Fill.OUT;
}

class FocusShapeSquare extends FocusShape{
  final double side;
  final double cornerRadius;
  FocusShapeSquare({double side, double cornerRadius, Fill fill}) :
    this.side = side ?? (MediaQueryData.fromWindow(window).size.shortestSide * 70 / 100),
    this.cornerRadius = cornerRadius ?? 16,
    super(Shape.SQUARE, fill);
}

class FocusShapeRectangle extends FocusShape{
  final double width, height;
  final double cornerRadius;
  FocusShapeRectangle({double width, double height, double cornerRadius, Fill fill}) :
    this.width = width ?? MediaQueryData.fromWindow(window).size.width * 70 / 100,
    this.height = height ?? MediaQueryData.fromWindow(window).size.height * 70 / 100,
    this.cornerRadius = cornerRadius ?? 16,
    super(Shape.RECTANGLE, fill);
}

class FocusShapeCircle extends FocusShape{
  final double radius;
  FocusShapeCircle({double radius, Fill fill}) :
    this.radius = radius ?? (MediaQueryData.fromWindow(window).size.shortestSide * 70 / 100) / 2,
    super(Shape.CIRCLE, fill);
}

class FocusOverlay extends StatelessWidget {

  final FocusShape focusShape;
  final Color color;
  Size size;
  Offset center;

  FocusOverlay({FocusShape focusShape, this.size, this.center, this.color = const Color(0x99000000)}) :
      this.focusShape = focusShape ?? FocusShapeSquare()
  ;

  @override
  Widget build(BuildContext context) {
    size ??= MediaQuery.of(context).size;
    return IgnorePointer(
      child: ClipPath(
        child: Container(
          width: size.width,
          height: size.height,
          color: color,
        ),
        clipper: _FocusCustomClipper(focusShape: focusShape, center: center),
      ),
    );
  }
}

class _FocusCustomClipper extends CustomClipper<Path> {
  final FocusShape focusShape;
  Offset center;
  _FocusCustomClipper({FocusShape focusShape, this.center}) :
        this.focusShape = focusShape ?? FocusShapeSquare()
  ;

  @override
  Path getClip(Size size) {
    var path = Path()..fillType = PathFillType.evenOdd;
    var centerX = center?.dx ?? size.width / 2;
    var centerY = center?.dy ?? size.height / 2;

    switch(focusShape.shape){
      case Shape.SQUARE:
        FocusShapeSquare square = focusShape;
        if(focusShape.fill == Fill.OUT)
          path.addRect(Rect.fromLTWH(0.0, 0.0, size.width, size.height));
        path.addRRect(
          RRect.fromRectAndRadius(
            Rect.fromCenter(
              center: Offset(centerX, centerY),
              width: square.side, height: square.side
            ),
            Radius.circular(square.cornerRadius)
          )
        );
        path.close();
        break;
      case Shape.RECTANGLE:
        FocusShapeRectangle rectangle = focusShape;
        if(focusShape.fill == Fill.OUT)
          path.addRect(Rect.fromLTWH(0.0, 0.0, size.width, size.height));
        path.addRRect(
            RRect.fromRectAndRadius(
                Rect.fromCenter(
                    center: Offset(centerX, centerY),
                    width: rectangle.width, height: rectangle.height
                ),
                Radius.circular(rectangle.cornerRadius)
            )
        );
        path.close();
        break;
      case Shape.CIRCLE:
        FocusShapeCircle circle = focusShape;
        var length = circle.radius;

        path.addOval(new Rect.fromCircle(
            center: new Offset(centerX, centerY), radius: length));
        if(focusShape.fill == Fill.OUT)
          path.addRect(Rect.fromLTWH(0.0, 0.0, size.width, size.height));
        path.close();
        break;
    }

    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return true;
  }
}